#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Class ShibaInit
# Shiba initialization class


from __future__ import unicode_literals

from shibaexceptions import *


class ShibaConnection(object):
    """The main Shiba class, standing as an initialization and mandatory for each submodules instancing.

        :param login: PriceMinister Seller login
        :param pwd: PriceMinister Seller Token \
        (see more at https://developer.priceminister.com/blog/fr/documentation/identification-by-token)
        :param sandbox: give it True if you want to test the API on a sandboxed version of PriceMinister.
        """

    def __init__(self, login, pwd, sandbox=False):
        self.login = str(login)
        self.pwd = str(pwd)
        self.domain = "https://ws.priceminister.com" if sandbox is False else "https://ws.sandbox.priceminister.com"
        self.actionsinfo = \
        {"producttypes": {"cat": "stock_ws", "version": "2011-11-29", "login": self.login, "pwd": self.pwd},
        "producttypetemplate": {"cat": "stock_ws", "version": "2013-05-14", "login": self.login, "pwd": self.pwd},
        "genericimportfile": {"cat": "stock_ws", "version": "2012-09-11", "login": self.login, "pwd": self.pwd},
        "genericimportreport": {"cat": "stock_ws", "version": "2011-11-29", "login": self.login, "pwd": self.pwd},
        "getavailableshippingtypes": {"cat": "sales_ws", "version": "2013-06-25", "login": self.login, "pwd": self.pwd},
        "export": {"cat": "stock_ws", "version": "2014-01-28", "login": self.login, "pwd": self.pwd},
        "listing": {"cat": "listing_ws", "version": "2014-01-28", "login": self.login, "pwd": self.pwd},
        "categorymap": {"cat": "categorymap_ws", "version": "2011-10-11", "login": self.login, "pwd": self.pwd},
        "getnewsales": {"cat": "sales_ws", "version": "2014-02-11", "login": self.login, "pwd": self.pwd},
        "acceptsale": {"cat": "sales_ws", "version": "2010-09-20", "login": self.login, "pwd": self.pwd},
        "refusesale": {"cat": "sales_ws", "version": "2010-09-20", "login": self.login, "pwd": self.pwd},
        "getcurrentsales": {"cat": "sales_ws", "version": "2014-02-11", "login": self.login, "pwd": self.pwd},
        "getbillinginformation": {"cat": "sales_ws", "version": "2011-03-29", "login": self.login, "pwd": self.pwd},
        "getshippinginformation": {"cat": "sales_ws", "version": "2014-02-11", "login": self.login, "pwd": self.pwd},
        "getitemtodolist": {"cat": "sales_ws", "version": "2011-09-01", "login": self.login, "pwd": self.pwd},
        "getiteminfos": {"cat": "sales_ws", "version": "2011-06-01", "login": self.login, "pwd": self.pwd},
        "cancelitem": {"cat": "sales_ws", "version": "2011-02-02", "login": self.login, "pwd": self.pwd},
        "contactusaboutitem": {"cat": "sales_ws", "version": "2011-09-01", "login": self.login, "pwd": self.pwd},
        "contactuseraboutitem": {"cat": "sales_ws", "version": "2011-02-02", "login": self.login, "pwd": self.pwd},
        "settrackingpackageinfos": {"cat": "sales_ws", "version": "2012-11-06", "login": self.login, "pwd": self.pwd},
        "confirmpreorder": {"cat": "sales_ws", "version": "2013-01-09", "login": self.login, "pwd": self.pwd},
        "getoperations": {"cat": "wallet_ws", "version": "2011-03-29", "login": self.login, "pwd": self.pwd},
        "getcompensationdetails": {"cat": "sales_ws", "version": "2011-03-29", "login": self.login, "pwd": self.pwd}}


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Class ShibaReturn
# Shiba return class

from __future__ import unicode_literals


class ShibaResponseObject(object):
    """This class gathers all content from the returned XML. Separating the namespace for further usage and the actual
    content. The raw XML from the WebServices is also reachable from this object.

    :param namespace: The namespace of the treated object, non really useful in our case but kept for further \
        development steps
    :param obj: The actual content of the returned XML, as an ObjectifiedElement from the lxml.objectify module
    :param xml: The raw XML returned by the WebServices

    Those arguments are stored into *content* for the ObjectifiedElement, *raw* for the raw and XML and *namespace* for
    the namespace of the XML, which have been removed.
    """

    def __init__(self, namespace, obj, xml):
        self.namespace = namespace
        self.content = obj
        self.rawxml = xml



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# class AccountingManagement


from __future__ import unicode_literals

from shibaconnection import ShibaConnection
from shibaexceptions import *
from shibatools import ShibaTools

from datetime import date


class AccountingManagement(object):
    """Accounting Management class, showing global financial operations on your account, or specific financial details
        about an operation"""
    def __init__(self, connection):
        assert(isinstance(connection, ShibaConnection)), "error : you must give this instance a ShibaConnection instance"
        self.connection = connection

    def get_operations(self, lastoperationdate=""):
        """Get global operations which happened on your wallet, compensationid given back from XML can be used
        in the get_compensation_details method below to get more detailed information about a specific operation.
        
        :param lastoperationdate: as follows : dd/mm/yyyy-hh:mm:ss and as string or date instance.
        """
        operationcause = "salestransfer"
        if isinstance(lastoperationdate, date) is False and type(lastoperationdate) is not str and \
                type(lastoperationdate) is not unicode:
            raise ShibaCallingError("Shiba code error : lastoperationdate parameter must be a datetime instance or str,"
                                    " got " + unicode(type(lastoperationdate)) + " instead.")
        if isinstance(lastoperationdate, date):
            lastoperationdate = lastoperationdate.strftime("%d/%m/%y-%H:%M:%S")
        inf = ShibaTools.inf_constructor(self.connection, "getoperations", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_compensation_details(self, compensationid):
        """Get a specific operation details from its "compensationid" found in the get_operation request return."""
        inf = ShibaTools.inf_constructor(self.connection, "getcompensationdetails", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Class ShibaTools
# Shiba runtime tools class


from __future__ import unicode_literals

import httplib
import urllib2 as ul2
import urllib as ul

from lxml import objectify
from lxml import etree

from shibaconnection import ShibaConnection
from shibaexceptions import *
from shibaresponseobject import ShibaResponseObject

import re

from collections import OrderedDict

import requests
import xmltodict


"""Tools used by Shiba data retrieving classes"""
class ShibaTools(object):
    @staticmethod
    def __errors_check(obj):
        """Errors checking from the returned XML as object. Handling errors as internal exceptions raising

        :param obj: objectified element to check errors from

        :rtype: objectified element if errors have been found, False if the WebService hasn't encountered an error
        """
        if "errorresponse" in obj.tag:
            if "ServerError" == obj.error.code:
                raise ShibaParameterError("Parameter error : " + obj.error.message +
                " - Reason : " + obj.error.details.detail)
            if "ParameterError" == obj.error.code:
                raise ShibaParameterError("Parameter error : " + obj.error.message +
                " - Reason : " + obj.error.details.detail)
            if "InvalidUserConnection" == obj.error.code:
                raise ShibaLoginError("Invalid user connection : " + obj.error.message +
                " - Reason : " + obj.error.details.detail)
            if "InvalidUserRights" == obj.error.code:
                raise ShibaRightsError("Invalid user rights : " + obj.error.message +
                " - Reason : " + obj.error.details.detail)
            return obj
        return False

    @staticmethod
    def post_request(url, data):
        """Method creating and submitting the multipart request of the wished to be imported XML file.

        :param data: raw XML as text

        :rtype: plain text from servers response
        """
        header = {"User-Agent": "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE; rv:1.9.0.10) "
                                        "Gecko/2009042316 Firefox/3.0.10 (.NET CLR 4.0.20506)"}
        data = data.encode('utf-8')
        d = {"file": data}
        r = requests.post(url, files=d, headers=header)
        return r.text


    @staticmethod
    def retrieve_obj_from_url(url, data=None):
        """Give it an URL, will send you back an object based on received XML from WebService, it removes and store
        namespace as its not really useful for common usages of this API.
        You can also give it a dict or an objectified tree to POST a file (used for import webservice)
        :param url: fully formatted URL from url_constructor
        :param data: if not None, will send a POST request to URL instead of a GET one, with multipart sending of
        this parameter content

        :rtype : ShibaResponseObject object, containing lxml.objectify class, raw XML as Unicode string and the
        namespace from the XML.
        """
        try:
            if data is not None:
                xml = ShibaTools.post_request(url, data)
            else:
                xml = ul2.urlopen(url).read()
        except requests.ConnectionError:
            raise ShibaConnectionError("HTTP error = Connection error - On URL: " + url)
        except ul2.HTTPError, e:
            raise ShibaConnectionError("HTTP error = " + unicode(e.code) + " - On URL: " + url)
        except ul2.URLError, e:
            raise ShibaConnectionError("URL error = " + unicode(e.reason) + " - On URL: " + url)
        except httplib.HTTPException:
            raise ShibaConnectionError("HTTP unknown error =" + " - On URL: " + url)
        xml = xml.decode('ISO-8859-1')
        try:
            namespace = re.search(pattern='xmlns="[^"]', string=xml)
            if namespace is not None:
                namespace = namespace.group()
            else:
                namespace = ""
            xmlepured = re.sub(pattern=' xmlns="[^"]+"', repl='', string=xml, flags=0)
            xmlepured = xmlepured.encode('utf-8')
            obj = objectify.fromstring(xmlepured)
        except:
            raise ShibaUnknownServiceError("Unknown error from service or from internal modules : Service returned : " + xml)
        if ShibaTools.__errors_check(obj) is not False:
            try:
                if "Unknown error" == obj.error.code:
                    raise ShibaServiceError("Unknown error from WebService (maybe the sale isn't confirmed yet?)"
                                            " : " + obj.error.message + " - Reason : " + obj.error.details.detail)
            except ShibaServiceError:
                raise ShibaServiceError("Unknown error from WebService (maybe the sale isn't confirmed yet?)"
                                        " : " + obj.error.message + " - Reason : " + obj.error.details.detail)
            except:
                raise ShibaUnknownServiceError("An unknown error from the WebService has occurred - XML dump : " +
                                                etree.tostring(obj))
        return ShibaResponseObject(namespace, obj, xml.encode('utf-8'))

    @staticmethod
    def create_xml_from_item_obj(inv):
        """Generate XML from the "inv" parameter.

        :param inv: is an object or a dict hierarchized as the XML structure described \
        in the WebServices documentation. Take a look at the xmltodict or the lxml.objectify documentation too.

        :rtype: string output from objectified element or ElementTree element tostring() methods
        """
        if type(inv) is etree._ElementTree:
            return etree.tostring(inv)
        elif type(inv) is dict or type(inv) is OrderedDict:
            try:
                return xmltodict.unparse(inv)
            except:
                raise ShibaCallingError("error from dictionary to xml : can't create XML from dictionary,"
                                        " refers to xmltodict documentation"
                                        "(you don't need to add a root element 'items' to your items)")
        else:
            raise ShibaCallingError("error : bad input parameter given, "
                                    "expecting dict, objectify object or ElementTree element")

    @staticmethod
    def inf_constructor(shibaconnection, action, **kwargs):
        """Just a selective concatenation of args given and related static data from the given action.

        :param shibaconnection: ShibaConnection class instance
        :param action: string giving the action type for the WebService
        :param kwargs: params to concatenate

        :rtype epured and updated dict
        """
        if isinstance(shibaconnection, ShibaConnection) is False:
            raise ShibaCallingError("Internal parameter error : shibaconnection parameter is not "
                                    "a ShibaConnection instance")
        if action not in shibaconnection.actionsinfo:
            raise ShibaCallingError("Internal parameter error : action parameter "
                                    + action + " is unknown from the actions list")
        newkwargs = {}
        for each in kwargs:
            if kwargs[each] is not None and kwargs[each] != "":
                newkwargs[each] = unicode(kwargs[each])
        newkwargs.update(shibaconnection.actionsinfo[action])
        newkwargs["action"] = action
        return newkwargs

    @staticmethod
    def url_constructor(shibaconnection, inf, domain=None):
        """URL constructor, formatting input and adding as many URL arguments as given.

        :param inf: info dict, from inf_constructor
        :param domain: force URL domain, useful for some actions needing to go with http instead of https

        :rtype: formatted URL string
        """
        if domain is None:
            domain = shibaconnection.domain
        pop = inf.pop("cat")
        if "self" in inf:
            inf.pop("self")
        primary = "%s/%s?" % (domain, pop)
        url = primary + ul.urlencode(inf)
        return url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# class InventoryManagement


from __future__ import unicode_literals

from shibaconnection import ShibaConnection
from shibaexceptions import *
from shibatools import ShibaTools


class InventoryManagement(object):
    """This class permits you to manage your inventory, get information about your products and even import products
    from XML to the PriceMinister platform"""
    def __init__(self, connection):
        if isinstance(connection, ShibaConnection) is False:
            raise ShibaCallingError("Shiba subclass init error : expecting a ShibaConnection instance")
        self.connection = connection

    def product_types(self):
        """This method retrieve products types from PriceMinister, helping you to define your products attributes."""
        inf = ShibaTools.inf_constructor(self.connection, "producttypes", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def product_type_template(self, alias, scope=""):
        """This methods retrieve product type attributes from the product type given through the "alias" parameter.
        Dedicated to help you making your own XML import file.

        :param alias: product alias as string
        :param scope: can be either VALUES or None, VALUES as "scope" retrieve attributes values instead of only \
        attributes
        """
        inf = ShibaTools.inf_constructor(self.connection, "producttypetemplate", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def generic_import_report(self, fileid, nexttoken=""):
        """Retrieves the report from a previous XML import, used as verification for a proper XML import

        :param fileid: import file ID given from the generic_import_file return
        :param nexttoken: used for pagination, can be used for a loop, its precised in the return from a first call \
        to this WebService
        """
        inf = ShibaTools.inf_constructor(self.connection, "genericimportreport", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def generic_import_file(self, data):
        """Import XML file to your PriceMinister inventory trough a POST request.

        :param data: must be a object/dict (OrderedDict is better) containing your inventory wished to be imported. \
        You must respect the XML hierarchy detailed from the WebService documentation inside the object/dict
        """
        data = ShibaTools.create_xml_from_item_obj(data)
        inf = ShibaTools.inf_constructor(self.connection, "genericimportfile")
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url, data)
        return obj

    def get_available_shipping_types(self):
        """Retrieves available shipping options at asking time"""
        inf = ShibaTools.inf_constructor(self.connection, "getavailableshippingtypes", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def export_inventory(self, scope="", nexttoken=""):
        """Export adverts from your inventory, nexttoken pagination is available.

        :param scope: PRICING is the only acceptable value for it, if not empty."""
        inf = ShibaTools.inf_constructor(self.connection, "export", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# class MarketplaceManagement


from __future__ import unicode_literals

from shibaconnection import ShibaConnection
from shibatools import ShibaTools
from shibaexceptions import *


class MarketplaceManagement(object):
    """ Marketplace informations retrieving, such as product lists and category mapping"""

    def __init__(self, connection):
        if isinstance(connection, ShibaConnection) is False:
            raise ShibaCallingError("Shiba subclass init error : expecting a ShibaConnection instance")
        self.connection = connection

    def get_product_list(self, scope="", kw="", nav="", refs="", productids="", nbproductsperpage="", pagenumber=""):
        """Prints a search list result from given parameters.

        :param scope: none (classic results), "PRICING" (classic results plus 10 best announces)
            or "LIMITED" (search in categories in which rapid put on sale is possible through WS)
        :param kw: research keyword
        :param nav: navigation category (url friendly ones, can be found on PriceMinister categories' URLs)
        :param refs: EAN, or ISBN, as a string, each value separated by a coma ','.
        :param productids: same as refs but as products ID.
        :param nbproductsperpage: products per page, default is 20.
        :param pagenumber: page number, default is 1.
        """
        if (type(refs) is not list and type(refs) is not str and type(refs) is not unicode) or \
                (type(productids) is not list and type(productids) is not str and type(productids) is not unicode):
            raise ShibaCallingError \
            ("Shiba code error : expected list or str/unicode as refs and/or productids parameters"
                ", got " + unicode(type(refs)) + " as refs and " + unicode(type(productids))
                + " as productids instead.")
        if type(refs) is list:
            refs = ','.join(refs)
        if type(productids) is list:
            productids = ','.join(productids)
        inf = ShibaTools.inf_constructor(self.connection, "listing", **locals())
        url = ShibaTools.url_constructor(self.connection, inf, domain="http://ws.priceminister.com")
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_category_map(self):
        """Lists items categories from the PriceMinister platform"""
        inf = ShibaTools.inf_constructor(self.connection, "categorymap", **locals())
        url = ShibaTools.url_constructor(self.connection, inf, domain="http://ws.priceminister.com")
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# class SalesManagement


from __future__ import unicode_literals

from shibaconnection import ShibaConnection
from shibatools import ShibaTools
from shibaexceptions import *

from datetime import date


class SalesManagement(object):
    """Primary sales management class, gather all sales-related methods."""
    def __init__(self, connection):
        if (isinstance(connection, ShibaConnection)) is False:
            raise ShibaCallingError("error : you must give this class a ShibaConnection instance")
        self.connection = connection

    def get_new_sales(self):
        """Calling get_new_sales all the new sales which have been recently recensed."""
        inf = ShibaTools.inf_constructor(self.connection, "getnewsales", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def accept_sale(self, itemid):
        """Accept the "itemid" sale.

        :param itemid: string for item ID
        """
        inf = ShibaTools.inf_constructor(self.connection, "acceptsale", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def refuse_sale(self, itemid):
        """Refuse the "itemid" sale.

        :param itemid: string for item ID
        """
        inf = ShibaTools.inf_constructor(self.connection, "refusesale", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_current_sales(self, ispendingpreorder="", purchasedate="", nexttoken=""):
        """Calling get_current_sales method gives you a obj gathering all current sales.

        :param ispendingpreorder: pass "y" is you want to see all preordered sales, leave empty as default
        :param purchasedate: Formatted as "yyyy-mm-dd" string allows you to filter the sales only from the given date
        :param nexttoken: Next page token argument, leave at is it, only give 0 is you want the first page
        """
        if ispendingpreorder != "" and ispendingpreorder != "y":
            raise ShibaCallingError("Shiba code error : ispendingpreorder parameter must be empty or 'y'")
        if isinstance(purchasedate, date) is False and type(purchasedate) is not str and \
                type(purchasedate) is not unicode:
            raise ShibaCallingError("Shiba code error : purchasedate order parameter must be a datetime "
                                    "instance or str, got " + unicode(type(purchasedate)) + " instead.")
        if isinstance(purchasedate, date):
            purchasedate = purchasedate.strftime("%d/%m/%y-%H:%M:%S")
        inf = ShibaTools.inf_constructor(self.connection, "getcurrentsales", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_billing_information(self, purchaseid):
        """Calling this method gives you accounting information about a confirmed order.

        :param purchaseid: is mandatory (same as found in get_new_sales report).
        """
        inf = ShibaTools.inf_constructor(self.connection, "getbillinginformation", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_shipping_information(self, purchaseid):
        """Quite similar to billing information method, but returns information about shipping for a given purchaseid
        Order forms are also available from this method return content.

        :param purchaseid: is mandatory (same as found in get_new_sales report).
        """
        inf = ShibaTools.inf_constructor(self.connection, "getshippinginformation", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_item_todo_list(self):
        """Retrieving a todo list on items, such as CLAIMS or MESSAGES from buyer or PriceMinister."""
        inf = ShibaTools.inf_constructor(self.connection, "getitemtodolist", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def get_item_infos(self, itemid):
        """This methods retrieves information from an "itemid" item, such as state, history, messages linked to it
        and actions available for this item. WARNING : All messages related to asked item will be tagged as "read".

        :param itemid: string for item ID
        """
        inf = ShibaTools.inf_constructor(self.connection, "getiteminfos", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def cancel_item(self, itemid, comment):
        """This method cancel the sale from the "itemid" item, after this one has been sold. WARNING : Cancelling
        a sale this way could harm your seller reputation. Use the "comment" param to specify the reason of this
        action to the related buyer (mandatory).

        :param itemid: string for item ID
        :param comment: message to send to the buyer as reason for cancelling the sale
        """
        inf = ShibaTools.inf_constructor(self.connection, "cancelitem", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def contact_us_about_item(self, itemid, content, mailparentid):
        """This functionality permits to join the PriceMinister after-sales service as buyer or seller.
        Specify the mailparentid to reply to a previous mail exchange. Message is message content, and itemid is
        the item PriceMinister ID related to the claim.

        :param itemid: string for item ID
        :param content: message content
        :param mailparentid: ID of previous mail, you must have conversed with the customer before to get it
        """
        inf = ShibaTools.inf_constructor(self.connection, "contactusaboutitem", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def contact_user_about_item(self, itemid, content):
        """Contact buyer of the "itemid" item in a regular way, sending him a message.

        :param itemid: string for item ID
        :param content: message content
        """
        inf = ShibaTools.inf_constructor(self.connection, "contactuseraboutitem", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def set_tracking_package_infos(self, itemid, transporter_name, tracking_number, tracking_url=""):
        """Send to buyer tracking information, such as the transporter's name "transporter_name",
        tracking number "tracking_number" and the optional tracking url "tracking_url".
        Please note that giving "Autre" as transporter_name brings the tracking url as mandatory.
        This WebService send an email to the "itemid" customer, including a link for package tracking.

        :param itemid: string for item ID
        :param transporter_name: transporter's name, string expected
        :param tracking_number: tracking number, preferably as string
        :param tracking_url: tracking URL for the package, as string, mandatory if transporter_name is 'Autre'
        """
        if transporter_name == "Autre" and len(tracking_url) == 0:
            raise ShibaCallingError("Shiba code error : if 'Autre' is specified as transporter_name, a tracking_url "
                                    "must be specified too")
        inf = ShibaTools.inf_constructor(self.connection, "settrackingpackageinfos", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj

    def confirm_preorder(self, advertid, stock):
        """Confirms preorders from the "advertid" item announce, will confirm "stock" items as confirmed orders for
        buyers who has preordered from the advert.

        :param advertid: advert ID as string
        :param stock: string or integer, must be positive
        """
        if int(stock) <= 0:
            raise ShibaCallingError("Shiba code error : stock must be a positive number")
        inf = ShibaTools.inf_constructor(self.connection, "confirmpreorder", **locals())
        url = ShibaTools.url_constructor(self.connection, inf)
        obj = ShibaTools.retrieve_obj_from_url(url)
        return obj


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# setup.py setup script

from setuptools import setup

setup(name= "Shiba",
      packages= ["shiba"],
      version = "1.1.1",
      description= "A Python API for PriceMinister WebServices",
      author= "Maxime Boguta",
      author_email= "maxime.boguta@epitech.eu",
      url= "https://github.com/ShibaAPI/shiba",
      download_url= "https://github.com/ShibaAPI/shiba/tarball/1.1.1",
      keywords= ["api", "priceminister", "python", "webservices"],
      install_requires=["requests", "xmltodict", "lxml"],)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Class ShibaException and inherited ones
# Shiba exceptions classes


from __future__ import unicode_literals

"""Those exception classes will usually be raised after an error returned from the WebServices"""
class   ShibaException(Exception):
    """Main exception class, as you can catch the whole of Shiba exceptions from it"""
    pass

class   ShibaParameterError(ShibaException):
    """Shiba parameter error, XML returned from the WebService told that one or more parameters are incorrect."""
    pass

class   ShibaLoginError(ShibaException):
    """Shiba login error, XML returned from the WebService told that the login informations given are incorrect."""
    pass

class   ShibaRightsError(ShibaException):
    """Shiba rights error, means XML returned the specified login IDs aren't authorized to access/modify such ressource
    from the asked WebService"""

class   ShibaServiceError(ShibaException):
    """Shiba service error, WebService told that an error have been encountered, but may can't precise where"""
    pass

class   ShibaUnknownServiceError(ShibaException):
    """Shiba unknown service error, WebService has caused/encoutered an error, but we can't know why (followed by a
    rough print of the XML retrieved"""
    pass

class   ShibaConnectionError(ShibaException):
    """Shiba connection error, kind of an internal error: URL can't be reached, in case of bad URL formatting or
    internet connection failure (HTTP errors 404, 500...)."""
    pass

class   ShibaCallingError(ShibaException):
    """Shiba internal error, for code relative errors. If such an exception is raised, look at given parameters to
    the API methods first as this is the major exception raising point for this type of exception"""
    pass

